#include "camera_node.h"
#include "utils/typeclass_helpers.h"
#include "utils/debug.h"

START_REFLECT(CameraNode);
REFLECT_TYPECLASS(CameraNode, Drop);
REFLECT_TYPECLASS(CameraNode, SceneNodeEntity);
END_REFLECT(CameraNode);

impl_Drop_for(CameraNode,
    DestroyCameraNode
)

impl_SceneNodeEntity_defaults(CameraNode);
impl_SceneNodeEntity_for(CameraNode,
    CameraNodeEnterTree,
    CameraNode_default_exit_tree,
    CameraNode_default_tick
)

SceneNode *CreateCameraNode() {
    CameraNode *self = new(CameraNode);
    return CreateSceneNode(CameraNode_as_SceneNodeEntity(self));
}

void DestroyCameraNode(CameraNode *self) {
    SceneNodeDetachEntity(self->node);
    free(self);
}

void CameraNodeEnterTree(CameraNode *self) {
    if(self->node->scene->main_camera == NULL)
        self->node->scene->main_camera = self;
    self->transform = TC_CAST(self->node->parent->entity, Transformable);
}

Camera3D CameraNodeGetCamera(CameraNode *self) {
    Transform global_transform = self->transform.tc->get_global_transform(self->transform.data);
    Matrix mat = TransformGetMatrix(&global_transform);
    Vector3 forward = { mat.m8, mat.m9, mat.m10 };
    return (Camera3D){
        .fovy = 90,
        .position = global_transform.translation,
        .projection = CAMERA_PERSPECTIVE,
        .target = Vector3Add(global_transform.translation, Vector3Scale(forward, 1000.f)),
        .up = (Vector3){mat.m4, mat.m5, mat.m6}
    };
}
